@@ -1,9 +1,12 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
+ * New Zealand.
+ *
  * Authors: Daniel Lawson 
- *          Perry Lorier 
+ *          Perry Lorier
+ *          Shane Alcock 
  *          
  * All rights reserved.
  *
@@ -24,16 +27,16 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_duck.c 1332 2008-03-25 21:45:36Z spa1 $
+ * $Id: format_duck.c 1644 2010-08-02 03:47:08Z salcock $
  *
  */
 
+#include "config.h"
 #include "libtrace.h"
 #include "libtrace_int.h"
 #include "format_helper.h"
-#include "config.h"
+#include "wandio.h"
 #include <stdlib.h>
-#include "libtraceio.h"
 #include "rt_protocol.h"
 
 #include <errno.h>
@@ -41,31 +44,39 @@
 #include <stdio.h>
 #include <fcntl.h>
 
+/* This format module deals with reading and writing DUCK records.
+ *
+ * Both DUCK record types (the DAG 2.4 and 2.5 versions) are supported by this
+ * module. 
+ *
+ * We differentiate between DUCK versions by writing the RT type to the start
+ * of the DUCK trace. This means that this code can only read DUCK files that
+ * were written using libtrace 3.
+ */
+
 #define DATA(x) ((struct duck_format_data_t *)x->format_data)
 #define DATAOUT(x) ((struct duck_format_data_out_t *)x->format_data)
 
-#define INPUT DATA(libtrace)
 #define OUTPUT DATAOUT(libtrace)
 
 struct duck_format_data_t {
 	char *path;
-	libtrace_io_t *file;
 	int dag_version;
 };
 
 struct duck_format_data_out_t {
 	char *path;
 	int level;
+	int compress_type;
 	int fileflag;
-	libtrace_io_t *file;
+	iow_t *file;
 	int dag_version;	
 };
 
 static int duck_init_input(libtrace_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct duck_format_data_t));
 
-	INPUT->file = 0;
-	INPUT->dag_version = 0;
+	DATA(libtrace)->dag_version = 0;
 	return 0;
 }
 
@@ -73,6 +84,7 @@ static int duck_init_output(libtrace_out_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct duck_format_data_out_t));
 	
 	OUTPUT->level = 0;
+	OUTPUT->compress_type = TRACE_OPTION_COMPRESSTYPE_NONE;
 	OUTPUT->fileflag = O_CREAT | O_WRONLY;
 	OUTPUT->file = 0;
 	OUTPUT->dag_version = 0;
@@ -86,6 +98,9 @@ static int duck_config_output(libtrace_out_t *libtrace,
 		case TRACE_OPTION_OUTPUT_COMPRESS:
 			OUTPUT->level = *(int *)data;
 			return 0;
+		case TRACE_OPTION_OUTPUT_COMPRESSTYPE:
+			OUTPUT->compress_type = *(int *)data;
+			return 0;
 		case TRACE_OPTION_OUTPUT_FILEFLAGS:
 			OUTPUT->fileflag = *(int *)data;
 			return 0;
@@ -99,19 +114,21 @@ static int duck_config_output(libtrace_out_t *libtrace,
 
 static int duck_start_input(libtrace_t *libtrace) {
 	
-	if (INPUT->file)
+	if (libtrace->io)
 		/* File already open */
 		return 0;
 	
-	INPUT->file = trace_open_file(libtrace);
-	if (!INPUT->file)
+	libtrace->io = trace_open_file(libtrace);
+	if (!libtrace->io)
 		return -1;
 
 	return 0;
 }
 
 static int duck_start_output(libtrace_out_t *libtrace) {
-	OUTPUT->file = trace_open_file_out(libtrace, OUTPUT->level,
+	OUTPUT->file = trace_open_file_out(libtrace, 
+						OUTPUT->compress_type,
+						OUTPUT->level,
 						OUTPUT->fileflag);
 	if (!OUTPUT->file) {
 		return -1;
@@ -120,14 +137,14 @@ static int duck_start_output(libtrace_out_t *libtrace) {
 }
 
 static int duck_fin_input(libtrace_t *libtrace) {
-	libtrace_io_close(INPUT->file);
+	wandio_destroy(libtrace->io);
 	free(libtrace->format_data);
 
 	return 0;
 }
 
 static int duck_fin_output(libtrace_out_t *libtrace) {
-	libtrace_io_close(OUTPUT->file);
+	wandio_wdestroy(OUTPUT->file);
 	free(libtrace->format_data);
 	return 0;
 }
@@ -177,9 +194,9 @@ static int duck_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 
 	flags |= TRACE_PREP_OWN_BUFFER;
 	
-	if (INPUT->dag_version == 0) {
+	if (DATA(libtrace)->dag_version == 0) {
 		/* Read in the duck version from the start of the trace */
-		if ((numbytes = libtrace_io_read(INPUT->file, &version, 
+		if ((numbytes = wandio_read(libtrace->io, &version, 
 					sizeof(version))) != sizeof(uint32_t)) {
 			trace_set_err(libtrace, errno, 
 					"Reading DUCK version failed");
@@ -188,24 +205,24 @@ static int duck_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		if (numbytes == 0) {
 			return 0;
 		}
-		INPUT->dag_version = bswap_le_to_host32(version);
+		DATA(libtrace)->dag_version = bswap_le_to_host32(version);
 	}
 	
 
-	if (INPUT->dag_version == TRACE_RT_DUCK_2_4) {
+	if (DATA(libtrace)->dag_version == TRACE_RT_DUCK_2_4) {
 		duck_size = sizeof(duck2_4_t);
 		packet->type = TRACE_RT_DUCK_2_4;
-	} else if (INPUT->dag_version == TRACE_RT_DUCK_2_5) {
+	} else if (DATA(libtrace)->dag_version == TRACE_RT_DUCK_2_5) {
 		duck_size = sizeof(duck2_5_t);
 		packet->type = TRACE_RT_DUCK_2_5;
 	} else {
 		trace_set_err(libtrace, TRACE_ERR_BAD_PACKET,
 				"Unrecognised DUCK version %i", 
-				INPUT->dag_version);
+				DATA(libtrace)->dag_version);
 		return -1;
 	}
 
-	if ((numbytes = libtrace_io_read(INPUT->file, packet->buffer,
+	if ((numbytes = wandio_read(libtrace->io, packet->buffer,
 					(size_t)duck_size)) != (int)duck_size) {
 		if (numbytes == -1) {
 			trace_set_err(libtrace, errno, "Reading DUCK failed");
@@ -231,6 +248,8 @@ static int duck_write_packet(libtrace_out_t *libtrace,
 {
 
 	int numbytes = 0;
+	uint32_t duck_version;
+
 	if (packet->type != TRACE_RT_DUCK_2_4 
 			&& packet->type != TRACE_RT_DUCK_2_5) {
 		trace_set_err_out(libtrace, TRACE_ERR_BAD_PACKET,
@@ -242,8 +261,9 @@ static int duck_write_packet(libtrace_out_t *libtrace,
 
 	if (OUTPUT->dag_version == 0) {
 	/* Writing the DUCK version will help with reading it back in later! */
-		if ((numbytes = libtrace_io_write(OUTPUT->file, &packet->type,
-				sizeof(packet->type))) != sizeof(uint32_t)){
+		duck_version = bswap_host_to_le32(packet->type);
+		if ((numbytes = wandio_wwrite(OUTPUT->file, &duck_version,
+				sizeof(duck_version))) != sizeof(uint32_t)){
 			trace_set_err_out(libtrace, errno, 
 					"Writing DUCK version failed");
 			return -1;
@@ -251,7 +271,7 @@ static int duck_write_packet(libtrace_out_t *libtrace,
 		OUTPUT->dag_version = packet->type;
 	}
 	
-	if ((numbytes = libtrace_io_write(OUTPUT->file, packet->payload, 
+	if ((numbytes = wandio_wwrite(OUTPUT->file, packet->payload, 
 					trace_get_capture_length(packet))) !=
 				(int)trace_get_capture_length(packet)) {
 		trace_set_err_out(libtrace, errno, "Writing DUCK failed");
@@ -301,8 +321,10 @@ static void duck_help(void) {
 }
 static struct libtrace_format_t duck = {
         "duck",
-        "$Id: format_duck.c 1332 2008-03-25 21:45:36Z spa1 $",
+        "$Id: format_duck.c 1644 2010-08-02 03:47:08Z salcock $",
         TRACE_FORMAT_DUCK,
+	NULL,				/* probe filename */
+	NULL,				/* probe magic */
         duck_init_input,	        /* init_input */
         NULL,                           /* config_input */
         duck_start_input,	        /* start_input */
@@ -321,6 +343,7 @@ static struct libtrace_format_t duck = {
         NULL,              		/* set_direction */
         NULL,          			/* get_erf_timestamp */
         NULL,                           /* get_timeval */
+	NULL,				/* get_timespec */
         NULL,                           /* get_seconds */
         NULL,                   	/* seek_erf */
         NULL,                           /* seek_timeval */
