@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_linux.c 1624 2010-07-27 05:04:51Z salcock $
+ * $Id: format_linux.c 1673 2010-11-30 21:33:40Z salcock $
  *
  */
 
@@ -174,6 +174,7 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 	if (FORMAT(libtrace->format_data)->fd==-1) {
 		trace_set_err(libtrace, errno, "Could not create raw socket");
 		free(libtrace->format_data);
+		libtrace->format_data = NULL;
 		return -1;
 	}
 
@@ -186,6 +187,7 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 			close(FORMAT(libtrace->format_data)->fd);
 			trace_set_err(libtrace, TRACE_ERR_INIT_FAILED, "Failed to find interface %s", libtrace->uridata);
 			free(libtrace->format_data);
+			libtrace->format_data = NULL;
 			return -1;
 		}
 	}
@@ -196,6 +198,7 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 				(struct sockaddr*)&addr,
 				(socklen_t)sizeof(addr))==-1) {
 		free(libtrace->format_data);
+		libtrace->format_data = NULL;
 		trace_set_err(libtrace, errno, "Failed to bind to interface %s", libtrace->uridata);
 		return -1;
 	}
@@ -304,10 +307,11 @@ static int linuxnative_pause_input(libtrace_t *libtrace)
 
 static int linuxnative_fin_input(libtrace_t *libtrace) 
 {
-	if (FORMAT(libtrace->format_data)->filter != NULL)
-		free(FORMAT(libtrace->format_data)->filter);
-	if (libtrace->format_data)
+	if (libtrace->format_data) {
+		if (FORMAT(libtrace->format_data)->filter != NULL)
+			free(FORMAT(libtrace->format_data)->filter);
 		free(libtrace->format_data);
+	}
 	
 	return 0;
 }
@@ -569,6 +573,9 @@ static int linuxnative_write_packet(libtrace_out_t *trace,
 {
 	struct sockaddr_ll hdr;
 
+	if (trace_get_link_type(packet) == TRACE_TYPE_NONDATA)
+		return 0;
+
 	hdr.sll_family = AF_PACKET;
 	hdr.sll_protocol = 0;
 	hdr.sll_ifindex = if_nametoindex(trace->uridata);
@@ -681,7 +688,14 @@ static int linuxnative_get_capture_length(const libtrace_packet_t *packet)
 
 static int linuxnative_get_wire_length(const libtrace_packet_t *packet) 
 {
-	return ((struct libtrace_linuxnative_header*)(packet->buffer))->wirelen;
+
+	int wirelen = ((struct libtrace_linuxnative_header*)(packet->buffer))->wirelen;
+
+	/* Include the missing FCS */
+	if (trace_get_link_type(packet) == TRACE_TYPE_ETH)
+		wirelen += 4;
+
+	return wirelen;
 }
 
 static int linuxnative_get_framing_length(UNUSED 
@@ -776,7 +790,7 @@ static uint64_t linuxnative_get_dropped_packets(libtrace_t *trace) {
 }
 
 static void linuxnative_help(void) {
-	printf("linuxnative format module: $Revision: 1624 $\n");
+	printf("linuxnative format module: $Revision: 1673 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tint:eth0\n");
 	printf("\n");
@@ -787,7 +801,7 @@ static void linuxnative_help(void) {
 }
 static struct libtrace_format_t linuxnative = {
 	"int",
-	"$Id: format_linux.c 1624 2010-07-27 05:04:51Z salcock $",
+	"$Id: format_linux.c 1673 2010-11-30 21:33:40Z salcock $",
 	TRACE_FORMAT_LINUX_NATIVE,
 	linuxnative_probe_filename,	/* probe filename */
 	NULL,				/* probe magic */
