@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: protocols_pktmeta.c 1646 2010-08-02 03:49:15Z salcock $
+ * $Id: protocols_pktmeta.c 1695 2011-03-10 03:24:22Z salcock $
  *
  */
 
@@ -36,17 +36,7 @@
 #include "protocols.h"
 #include <assert.h>
 
-#ifndef WIN32
-#include <net/if_arp.h>
-#endif
-
-#ifndef ARPHRD_ETHER
-#define ARPHRD_ETHER    1               /* Ethernet 10/100Mbps.  */
-#endif
-
-#ifndef ARPHRD_PPP
-#define ARPHRD_PPP      512
-#endif
+#include "arphrd.h"
 
 /* This file contains all the protocol decoding functions for the meta-data
  * headers that may be prepended to captured packets.
@@ -60,7 +50,8 @@
 
 /* NB: type is returned as an ARPHRD_ type for SLL*/
 void *trace_get_payload_from_linux_sll(const void *link,
-		uint16_t *type, uint32_t *remaining) 
+		uint16_t *arphrd_type, uint16_t *next, 
+		uint32_t *remaining) 
 {
 	libtrace_sll_header_t *sll;
 
@@ -74,7 +65,11 @@ void *trace_get_payload_from_linux_sll(const void *link,
 		*remaining-=sizeof(*sll);
 	}
 
-	if (type) *type = ntohs(sll->hatype);
+	/* The SLL header is actually in place of a link layer header, so
+	 * we want to use the protocol field to tell our caller what the
+	 * next header is going to be */
+	if (next) *next = (libtrace_linktype_t)(ntohs(sll->protocol));
+	if (arphrd_type) *arphrd_type = ntohs(sll->hatype);
 
 	return (void*)((char*)sll+sizeof(*sll));
 
@@ -180,6 +175,7 @@ DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
 {
 	void *nexthdr; 
 	uint16_t arphrd;
+	uint16_t next;
 	
 	assert(meta != NULL);
 	assert(linktype != NULL);
@@ -188,8 +184,14 @@ DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
 	switch(*linktype) {
 		case TRACE_TYPE_LINUX_SLL:
 			nexthdr = trace_get_payload_from_linux_sll(meta,
-					&arphrd, remaining);
-			*linktype = arphrd_type_to_libtrace(arphrd);
+					&arphrd, &next, remaining);
+
+			/* Ethernet header is usually absent in SLL captures,
+			 * so we don't want to skip it just yet */
+			if (arphrd_type_to_libtrace(arphrd) == TRACE_TYPE_ETH && next != 0x0060) 
+				*linktype = TRACE_TYPE_NONE; 
+			else 
+				*linktype = arphrd_type_to_libtrace(arphrd);
 			return nexthdr;
 		case TRACE_TYPE_80211_RADIO:
 			nexthdr = trace_get_payload_from_radiotap(meta,
