@@ -16,6 +16,8 @@ static bitbuffer_t getbit(void **packet, int *packlen, uint64_t numbits)
 {
     bitbuffer_t ret;
     bitbuffer_t mask;
+    
+    char *pktptr = NULL;
 
     /* While the buffer is not filled up and there is still
      * data in the packet to read, read a byte...
@@ -29,7 +31,11 @@ static bitbuffer_t getbit(void **packet, int *packlen, uint64_t numbits)
 	byte=(*((bitbuffer_t*)*packet))&0xff;
 	buffer |= (bitbuffer_t)byte << (sizeof(bitbuffer_t)*8-(bits+sizeof(byte)*8));
 	/* update the position within the packet */
-	*packet = ((char*)*packet) + 1;
+	pktptr = (char *)*packet;
+	pktptr += 1;
+	*packet = pktptr;
+
+	//*packet = ((char*)*packet) + 1;
 
 	bits += sizeof(byte)*8;
 	*packlen -= 1;
@@ -49,6 +55,8 @@ static bitbuffer_t getbit(void **packet, int *packlen, uint64_t numbits)
     return ret;
 }
 
+int yyparse(void);
+
 element_t* parse_protocol_file(char *filename)
 {
     /* hold onto this so we can put it in any error messages */
@@ -108,7 +116,7 @@ static bitbuffer_t fix_byteorder(bitbuffer_t value,
 
 
 
-void decode_protocol_file(uint16_t link_type,const char *packet,int len,element_t *el)
+void decode_protocol_file(uint16_t link_type UNUSED,const char *packet,int len,element_t *el)
 {
     bitbuffer_t result;
 
@@ -133,7 +141,7 @@ void decode_protocol_file(uint16_t link_type,const char *packet,int len,element_
 				el->data->field->size);
 				
 			el->data->field->value = result;
-			printf(" %s %lld\n", 
+			printf(" %s %" PRIi64 "\n", 
 				el->data->field->identifier,
 				result);
 		    }
@@ -150,7 +158,7 @@ void decode_protocol_file(uint16_t link_type,const char *packet,int len,element_
 				el->data->field->size);
 			
 			el->data->field->value = result;
-			printf(" %s 0x%llx\n", 
+			printf(" %s 0x%" PRIx64 "\n", 
 				el->data->field->identifier,
 				result);
 		    }
