@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: tracestats.c 1339 2008-04-02 03:34:39Z spa1 $
+ * $Id: tracestats.c 1486 2010-01-18 22:59:23Z perry $
  *
  */
 
@@ -61,6 +61,7 @@ static volatile int done=0;
 
 static void cleanup_signal(int signal)
 {
+	(void)signal;
 	done=1;
 }
 
@@ -88,12 +89,12 @@ static void run_trace(char *uri)
         trace = trace_create(uri);
 
 	if (trace_is_err(trace)) {
-		trace_perror(trace,"");
+		trace_perror(trace,"Failed to create trace");
 		return;
 	}
 
 	if (trace_start(trace)==-1) {
-		trace_perror(trace,"");
+		trace_perror(trace,"Failed to start trace");
 		return;
 	}
 
@@ -151,6 +152,9 @@ static void run_trace(char *uri)
 	totcount+=count;
 	totbytes+=bytes;
 
+	if (trace_is_err(trace))
+		trace_perror(trace,"%s",uri);
+
         trace_destroy(trace);
 }
 
